@@ -1,5 +1,8 @@
 #![no_std]
 #![deny(warnings)]
+#![doc(test(attr(deny(warnings))))]
+#![doc(test(attr(allow(dead_code))))]
+#![doc(test(attr(allow(unused_variables))))]
 
 #[doc(hidden)]
 pub use core::compile_error as std_compile_error;
@@ -160,7 +163,24 @@ macro_rules! parse_generics_impl {
         [ : $($token:tt)*]
     ) => {
         $crate::parse_generics_impl! {
-            @constrained_param
+            @constrained_param [:]
+            [$param]
+            []
+            [$callback] [$($callback_args)*] [$($g)*] [$($r)*]
+            [$($token)*]
+        }
+    };
+    (
+        @param
+        [$param:tt]
+        [$callback:path]
+        [$($callback_args:tt)*]
+        [$($g:tt)*]
+        [$($r:tt)*]
+        [ = $($token:tt)*]
+    ) => {
+        $crate::parse_generics_impl! {
+            @constrained_param [=]
             [$param]
             []
             [$callback] [$($callback_args)*] [$($g)*] [$($r)*] 
@@ -250,7 +270,7 @@ macro_rules! parse_generics_impl {
         ));
     };
     (
-        @constrained_param
+        @constrained_param [$kind:tt]
         [$param:tt]
         [$($constraint:tt)*]
         [$callback:path]
@@ -260,7 +280,7 @@ macro_rules! parse_generics_impl {
         [ < $($token:tt)*]
     ) => {
         $crate::parse_generics_impl! {
-            @angles_in_constraint
+            @angles_in_constraint [$kind]
             [$param]
             [$($constraint)*]
             [] []
@@ -269,7 +289,7 @@ macro_rules! parse_generics_impl {
         }
     };
     (
-        @constrained_param
+        @constrained_param [:]
         [$param:tt]
         [$($constraint:tt)*]
         [$callback:path]
@@ -288,7 +308,26 @@ macro_rules! parse_generics_impl {
         }
     };
     (
-        @constrained_param
+        @constrained_param [=]
+        [$param:tt]
+        [$($constraint:tt)*]
+        [$callback:path]
+        [$($callback_args:tt)*]
+        [$($g:tt)*]
+        [$($r:tt)*]
+        [ > $($token:tt)*]
+    ) => {
+        $crate::parse_generics_impl! {
+            @done
+            [$callback] [$($callback_args)*]
+            [$($g)* [$param]]
+            [$($r)* [$param]]
+            []
+            [$($token)*]
+        }
+    };
+    (
+        @constrained_param [:]
         [$param:tt]
         [$($constraint:tt)*]
         [$callback:path]
@@ -307,7 +346,26 @@ macro_rules! parse_generics_impl {
         }
     };
     (
-        @constrained_param
+        @constrained_param [=]
+        [$param:tt]
+        [$($constraint:tt)*]
+        [$callback:path]
+        [$($callback_args:tt)*]
+        [$($g:tt)*]
+        [$($r:tt)*]
+        [ , > $($token:tt)*]
+    ) => {
+        $crate::parse_generics_impl! {
+            @done
+            [$callback] [$($callback_args)*]
+            [$($g)* [$param]]
+            [$($r)* [$param]]
+            []
+            [$($token)*]
+        }
+    };
+    (
+        @constrained_param [:]
         [$param:tt]
         [$($constraint:tt)*]
         [$callback:path]
@@ -324,7 +382,24 @@ macro_rules! parse_generics_impl {
         }
     };
     (
-        @constrained_param
+        @constrained_param [=]
+        [$param:tt]
+        [$($constraint:tt)*]
+        [$callback:path]
+        [$($callback_args:tt)*]
+        [$($g:tt)*]
+        [$($r:tt)*]
+        [ , $($token:tt)*]
+    ) => {
+        $crate::parse_generics_impl! {
+            [$callback] [$($callback_args)*]
+            [$($g)* [$param]]
+            [$($r)* [$param]]
+            [$($token)*]
+        }
+    };
+    (
+        @constrained_param [$kind:tt]
         [$param:tt]
         [$($constraint:tt)*]
         [$callback:path]
@@ -334,7 +409,7 @@ macro_rules! parse_generics_impl {
         [ $x:tt $($token:tt)*]
     ) => {
         $crate::parse_generics_impl! {
-            @constrained_param
+            @constrained_param [$kind]
             [$param]
             [$($constraint)* $x]
             [$callback] [$($callback_args)*] [$($g)*] [$($r)*] 
@@ -342,7 +417,7 @@ macro_rules! parse_generics_impl {
         }
     };
     (
-        @constrained_param
+        @constrained_param [$kind:tt]
         [$param:tt]
         [$($constraint:tt)*]
         [$callback:path]
@@ -353,12 +428,12 @@ macro_rules! parse_generics_impl {
     ) => {
         $crate::std_compile_error!($crate::std_concat!(
             "missing '>' after '",
-            $crate::std_stringify!( < $($($($g)*),+ ,)? $param : $($constraint)* ),
+            $crate::std_stringify!( < $($($($g)*),+ ,)? $param $kind $($constraint)* ),
             "'"
         ));
     };
     (
-        @angles_in_constraint
+        @angles_in_constraint [$kind:tt]
         [$param:tt]
         [$($constraint:tt)*]
         [$($inside_angles:tt)*]
@@ -370,7 +445,7 @@ macro_rules! parse_generics_impl {
         [ > $($token:tt)*]
     ) => {
         $crate::parse_generics_impl! {
-            @constrained_param
+            @constrained_param [$kind]
             [$param]
             [$($constraint)* < $($inside_angles)* > ]
             [$callback] [$($callback_args)*] [$($g)*] [$($r)*] 
@@ -378,7 +453,7 @@ macro_rules! parse_generics_impl {
         }
     };
     (
-        @angles_in_constraint
+        @angles_in_constraint [$kind:tt]
         [$param:tt]
         [$($constraint:tt)*]
         [$($inside_angles:tt)*]
@@ -390,7 +465,7 @@ macro_rules! parse_generics_impl {
         [ > $($token:tt)*]
     ) => {
         $crate::parse_generics_impl! {
-            @angles_in_constraint
+            @angles_in_constraint [$kind]
             [$param] [$($constraint)*]
             [$($parent_level)* < $($inside_angles)* > ]
             [$([$($outer_levels)*])*]
@@ -399,7 +474,7 @@ macro_rules! parse_generics_impl {
         }
     };
     (
-        @angles_in_constraint
+        @angles_in_constraint [$kind:tt]
         [$param:tt]
         [$($constraint:tt)*]
         [$($inside_angles:tt)*]
@@ -411,7 +486,7 @@ macro_rules! parse_generics_impl {
         [ < $($token:tt)*]
     ) => {
         $crate::parse_generics_impl! {
-            @angles_in_constraint
+            @angles_in_constraint [$kind]
             [$param] [$($constraint)*]
             []
             [[$($inside_angles:tt)*] $([$($outer_levels)*])*]
@@ -420,7 +495,7 @@ macro_rules! parse_generics_impl {
         }
     };
     (
-        @angles_in_constraint
+        @angles_in_constraint [$kind:tt]
         [$param:tt]
         [$($constraint:tt)*]
         [$($inside_angles:tt)*]
@@ -432,7 +507,7 @@ macro_rules! parse_generics_impl {
         [$x:tt $($token:tt)*]
     ) => {
         $crate::parse_generics_impl! {
-            @angles_in_constraint
+            @angles_in_constraint [$kind]
             [$param] [$($constraint)*]
             [$($inside_angles)* $x]
             [$([$($outer_levels)*])*]
@@ -441,7 +516,7 @@ macro_rules! parse_generics_impl {
         }
     };
     (
-        @angles_in_constraint
+        @angles_in_constraint [$kind:tt]
         [$param:tt]
         [$($constraint:tt)*]
         [$($inside_angles:tt)*]
@@ -455,7 +530,7 @@ macro_rules! parse_generics_impl {
         $crate::std_compile_error!($crate::std_concat!(
             "missing '>' after '",
             $crate::std_stringify!(
-                < $($($($g)*),+ ,)? $param : $($constraint)*
+                < $($($($g)*),+ ,)? $param $kind $($constraint)*
                 $( < $($outer_levels)* )* < $($inside_angles)*
             ),
             "'"
@@ -1261,6 +1336,12 @@ mod tests {
         struct TestGenericStruct<'a, T: 'static> { }
     }
 
+    struct TestGenericStructWithDefaultParameter<T=()>(T);
+
+    impl_test_trait! {
+        struct TestGenericStructWithDefaultParameter<T=()>(T);
+    }
+
     #[test]
     fn it_works() {
         let test_struct = TestStruct { };
@@ -1272,6 +1353,10 @@ mod tests {
         let _ = test_generic_struct.a;
         let _ = test_generic_struct.t;
         let _: &dyn TestTrait = &test_generic_struct;
+        let test_generic_struct_ = TestGenericStructWithDefaultParameter(());
+        let _: &dyn TestTrait = &test_generic_struct_;
+        let _ = test_generic_struct.a;
+        let _ = test_generic_struct.t;
     }
 
     macro_rules! impl_tr {
